@@ -44,11 +44,12 @@
 (require 'xcb)
 (require 'xcb-xkb)
 
-(defclass xcb:keysyms:-device ()
+(defclass xcb:keysyms:-device (xcb:--struct)
   ((keytypes :initform nil)
    (keycodes :initform nil)
    (min-keycode :initform 0)
-   (max-keycode :initform 0))
+   (max-keycode :initform 0)
+   (updated :initform nil))
   :documentation "Device (keyboard) properties.")
 
 ;; These variables are shared by all connections.
@@ -150,26 +151,21 @@ This method must be called before using any other method in this module."
 (cl-defmethod xcb:keysyms:-on-NewKeyboardNotify ((obj xcb:connection) data)
   "Handle 'NewKeyboardNotify' event."
   (let ((device-id (xcb:-get-extra-plist obj 'keysyms 'device-id))
-        (opcode (xcb:-get-extra-plist obj 'keysyms 'opcode))
         (callback (xcb:-get-extra-plist obj 'keysyms 'callback))
         (obj1 (make-instance 'xcb:xkb:NewKeyboardNotify))
-        updated)
+        device updated)
     (xcb:unmarshal obj1 data)
     (with-slots (deviceID oldDeviceID requestMajor requestMinor changed) obj1
       (if (= 0 (logand changed xcb:xkb:NKNDetail:DeviceID))
-          (when (and (/= 0 (logand changed xcb:xkb:NKNDetail:Keycodes))
-                     ;; Also, according to the specification this can
-                     ;; only happen when a GetKbdByName request issued.
-                     ;; The two checks below avoid false positive caused
-                     ;; by requests such as SetMap (e.g. XCape).
-                     (= requestMajor opcode)
-                     (= requestMinor
-                        (eieio-oref-default 'xcb:xkb:GetKbdByName '~opcode)))
-            ;; (xcb:keysyms:-update-keytypes obj deviceID)
-            (xcb:keysyms:-update-keycodes obj deviceID)
-            (when (= deviceID device-id)
-              (setq updated t)
-              (xcb:keysyms:-update-modkeys obj deviceID)))
+          (when (/= 0 (logand changed xcb:xkb:NKNDetail:Keycodes))
+            (setq device (xcb:-get-extra-plist obj 'keysyms deviceID))
+            (when (and device
+                       (not (slot-value device 'updated)))
+              (xcb:keysyms:-update-keycodes obj deviceID)
+              (when (= deviceID device-id)
+                (setq updated t)
+                (xcb:keysyms:-update-modkeys obj deviceID))
+              (setf (slot-value device 'updated) t)))
         (xcb:keysyms:-set-per-client-flags obj deviceID)
         (xcb:keysyms:-update-keytypes obj deviceID)
         (xcb:keysyms:-update-keycodes obj deviceID)
@@ -400,7 +396,7 @@ Return (0 . 0) when conversion fails."
           (pcase (logand group-info #xC0) ;See <XKBstr.h>.
             (`xcb:xkb:GroupsWrap:RedirectIntoRange
              (setq group (logand #xFF (lsh group-info -4))) ;See <XKBstr.h>.
-             ;; Check if i's also out of range.
+             ;; Check if it's also out of range.
              (when (>= group group-number)
                (setq group 0)))
             (`xcb:xkb:GroupsWrap:ClampIntoRange
