@@ -51,16 +51,25 @@
 (eval-when-compile (require 'cl-lib))
 (require 'cl-generic)
 (require 'eieio)
+(require 'xcb-debug)
 
-(eval-when-compile
-  (defvar xcb:debug-on nil "Non-nil to turn on debug."))
+(define-minor-mode xcb:debug
+  "Debug-logging enabled if non-nil"
+  :global t)
 
-(defmacro xcb:-log (format-string &rest args)
-  "Print debug info."
-  (when xcb:debug-on
-    `(message (concat "[XELB LOG] " ,format-string) ,@args)))
+(defmacro xcb:-log (&optional format-string &rest objects)
+  "Emit a message prepending the name of the function being executed.
 
-;;;; Fix backward compatibility issues with Emacs < 25
+FORMAT-STRING is a string specifying the message to output, as in
+`format'.  The OBJECTS arguments specify the substitutions."
+  (unless format-string (setq format-string ""))
+  `(when xcb:debug
+     (xcb-debug:message ,(concat "%s:\t" format-string "\n")
+                        (xcb-debug:compile-time-function-name)
+                        ,@objects)
+     nil))
+
+;;;; Fix backward compatibility issues with Emacs 24
 
 (eval-and-compile
   (when (< emacs-major-version 25)
@@ -78,22 +87,21 @@
                       spec-list)
            ,@body)))))
 
-;; Backport some new functions from Emacs 25
+;; Backport some functions to Emacs 24
 
 (eval-and-compile
   (unless (fboundp 'eieio-class-slots)
-    (eval-and-compile
-      (defun eieio-class-slots (class)
-        (let* ((tmp (class-v class))
-               (names (eieio--class-public-a tmp))
-               (initforms (eieio--class-public-d tmp))
-               (types (eieio--class-public-type tmp))
-               result)
-          (dotimes (i (length names))
-            (setq result (nconc result (list (vector (elt names i)
-                                                     (elt initforms i)
-                                                     (elt types i))))))
-          result)))))
+    (defun eieio-class-slots (class)
+      (let* ((tmp (get class 'eieio-class-definition))
+             (names (aref tmp 5))
+             (initforms (aref tmp 6))
+             (types (aref tmp 8))
+             result)
+        (dotimes (i (length names))
+          (setq result (nconc result (list (vector (elt names i)
+                                                   (elt initforms i)
+                                                   (elt types i))))))
+        result))))
 
 (eval-and-compile
   (unless (fboundp 'eieio-slot-descriptor-name)
@@ -403,10 +411,16 @@
 
 ;; typedef in C
 (defmacro xcb:deftypealias (new-type old-type)
-  "Define NEW-TYPE as an alias of type OLD-TYPE."
+  "Define NEW-TYPE as an alias of type OLD-TYPE.
+
+Also the fundamental type is stored in 'the xcb--typealias' variable
+property (for internal use only)."
   `(progn
-     (cl-deftype ,(eval new-type) nil ,old-type)
-     (defvaralias ,new-type ,old-type)))
+     ;; FIXME: `new-type' should probably just not be eval'd at all,
+     ;; but that requires changing all callers not to quote their arg.
+     (cl-deftype ,(eval new-type t) nil ,old-type)
+     (put ,new-type 'xcb--typealias
+          (or (get ,old-type 'xcb--typealias) ,old-type))))
 
 ;; 1/2/4 B signed/unsigned integer
 (cl-deftype xcb:-i1 () t)
@@ -454,10 +468,10 @@ Consider let-bind it rather than change its global value."))
   nil)
 
 (cl-defmethod slot-unbound ((object xcb:--struct) class slot-name fn)
-  (xcb:-log "unbount-slot: %s" (list (eieio-class-name class)
-                                     (eieio-object-name object)
-			             slot-name fn))
-  nil)
+  (unless (eq fn #'oref-default)
+    (xcb:-log "unbound-slot: %s" (list (eieio-class-name class)
+                                       (eieio-object-name object)
+			               slot-name fn))))
 
 (defclass xcb:-struct (xcb:--struct)
   ((~lsb :initarg :~lsb
@@ -490,7 +504,7 @@ and value VALUE.
 
 The optional POS argument indicates current byte index of the field (used by
 `xcb:-pad-align' type)."
-  (pcase (indirect-variable type)
+  (pcase (or (get type 'xcb--typealias) type)
     (`xcb:-u1 (xcb:-pack-u1 value))
     (`xcb:-i1 (xcb:-pack-i1 value))
     (`xcb:-u2
@@ -601,7 +615,7 @@ The optional argument CTX is for <paramref>.
 
 This method returns a list of two components, with the first being the result
 and the second the consumed length."
-  (pcase (indirect-variable type)
+  (pcase (or (get type 'xcb--typealias) type)
     (`xcb:-u1 (list (aref data offset) 1))
     (`xcb:-i1 (let ((result (aref data offset)))
                 (list (if (< result 128) result (- result 255)) 1)))
@@ -780,6 +794,9 @@ This method auto-pads short results to 32 bytes."
   ((~size :initarg :~size :type xcb:-ignore)) ;Size of the largest member.
   :documentation "Union type.")
 ;;
+(cl-defmethod slot-unbound ((_object xcb:-union) _class _slot-name _fn)
+  nil)
+;;
 (cl-defmethod xcb:marshal ((obj xcb:-union))
   "Return the byte-array representation of union OBJ.
 
