@@ -44,10 +44,6 @@
 ;;   - `xcb:generate-id'
 ;; Please refer to their documentations for more details.
 
-;; If you ever encountered errors when using this library, please set
-;; `xcb:debug-on' to `t' before loading it; this should provide you more clues
-;; on what is going wrong.
-
 ;; Todo:
 ;; + Use XC-MISC extension for `xcb:generate-id' when IDs are used up.
 
@@ -395,18 +391,23 @@ Concurrency is disabled as it breaks the orders of errors, replies and events."
                 (substring message-cache (- cache-length (length cache))))
           (when (/= current-cache-length cache-length)
             (xcb:-connection-filter process []))))
-      (with-slots (event-lock event-queue) connection
-        (unless (< 0 event-lock)
-          (cl-incf event-lock)
-          (unwind-protect
-              (let (event data synthetic)
-                (while (setq event (pop event-queue))
-                  (setq data (aref event 1)
-                        synthetic (aref event 2))
-                  (dolist (listener (aref event 0))
-                    (with-demoted-errors "[XELB ERROR] %S"
-                      (funcall listener data synthetic)))))
-            (cl-decf event-lock)))))))
+      (xcb:-process-events connection))))
+
+(cl-defmethod xcb:-process-events ((conn xcb:connection))
+  "Process cached events."
+  (with-slots (event-lock event-queue) conn
+    (unless (< 0 event-lock)
+      (cl-incf event-lock)
+      (unwind-protect
+          (let (event data synthetic)
+            (while (setq event (pop event-queue))
+              (setq data (aref event 1)
+                    synthetic (aref event 2))
+              (dolist (listener (aref event 0))
+                (unwind-protect
+                    (xcb-debug:backtrace-on-error
+                     (funcall listener data synthetic))))))
+        (cl-decf event-lock)))))
 
 (cl-defmethod xcb:disconnect ((obj xcb:connection))
   "Disconnect from X server."
@@ -464,7 +465,8 @@ classes of EVENT (since they have the same event number)."
       (unwind-protect
           (process-send-string (slot-value obj 'process)
                                (apply #'unibyte-string (append cache nil)))
-        (cl-decf (slot-value obj 'event-lock))))))
+        (cl-decf (slot-value obj 'event-lock)))
+      (xcb:-process-events obj))))
 
 (cl-defmethod xcb:get-extension-data ((obj xcb:connection) namespace)
   "Fetch the extension data from X server (block until data is retrieved)."
@@ -559,7 +561,7 @@ classes of EVENT (since they have the same event number)."
               last-seen-sequence 0))
       (setf request-cache (vconcat cache msg)
             request-sequence (1+ request-sequence))
-      (xcb:-log "Cache request #%d: %s" request-sequence request)
+      (xcb:-log "Cache request #%d: %s" request-sequence msg)
       request-sequence)))
 
 (cl-defmethod xcb:-+request ((obj xcb:connection) request)
@@ -627,7 +629,8 @@ Otherwise no error will ever be reported."
             (while (and (> sequence (slot-value obj 'last-seen-sequence))
                         (<= sequence (slot-value obj 'request-sequence)))
               (accept-process-output process 1 nil 1)))
-        (cl-decf (slot-value obj 'event-lock)))))
+        (cl-decf (slot-value obj 'event-lock)))
+      (xcb:-process-events obj)))
   (let* ((reply-plist (slot-value obj 'reply-plist))
          (reply-data (plist-get reply-plist sequence))
          (error-plist (slot-value obj 'error-plist))
@@ -742,6 +745,7 @@ Sync by sending a GetInputFocus request and waiting until it's processed."
                       (<= sequence (slot-value obj 'request-sequence)))
             (accept-process-output process 1 nil 1)))
       (cl-decf (slot-value obj 'event-lock)))
+    (xcb:-process-events obj)
     ;; Discard any reply or error.
     (cl-remf (slot-value obj 'reply-plist) sequence)
     (cl-remf (slot-value obj 'error-plist) sequence)))
